indexing

   description: "Class that covers a POSIX directory. Main use is to be %
   %able to loop through a single directory."

   author: "Berend de Boer"
   date: "$Date: Fri, 13 Oct 2000 13:36:48 +0200 $";
   revision: "$Revision: 1.3 $"

class 

   POSIX_DIRECTORY

inherit

   POSIX_BASE

   MEMORY
      redefine
         dispose
      end

   PAPI_DIRENT
   

creation
   
   make
   

feature -- creation

   make (a_directory_name: STRING) is
      do
         create directory_name.make (a_directory_name)
         create item.make(max_filename_length)
      end


feature -- access   

   close is
         -- close directory entry (save resources now, don't wait for
         -- garbage collection). If you call start it will automatically
         -- reopen
      require
         not_closed: is_open
      local
         r: INTEGER
      do
         r := posix_closedir (dirp)
         if r = -1 then
            raise_posix_error
         end
         dirp := default_pointer
      ensure
         closed: not is_open
      end   
   
   start is
      -- start directory traversal
      do
         exhausted := False
         item.resize ( max_filename_length)
         if dirp = default_pointer then
            dirp := posix_opendir (sh.string_to_pointer(directory_name))
            if dirp = default_pointer then
               raise_posix_error
            end
         else            
            posix_rewinddir (dirp)
         end
         forth
      end
   
   forth is
      -- go to next entry
      require
         opened: is_open
         not_exhausted: not exhausted
      local
         dirent: POINTER
      do
         errno.clear
         dirent := posix_readdir (dirp)
         if dirent = default_pointer then
            if errno.is_not_ok then
               raise_posix_error
            else
               exhausted := True
               item.wipe_out
            end
         else
            sh.set_string_from_pointer(item, posix_d_name (dirent))
         end
      end
   
   item: STRING
         -- the current entry


feature -- status report

   exhausted: BOOLEAN
         -- no more entries in this directory
   
   is_empty: BOOLEAN

   is_first: BOOLEAN
         -- current item is first entry 

   is_open: BOOLEAN is
         -- True if directory is ready for traversal
      do
         Result := dirp /= default_pointer
      end


feature
   
   max_filename_length: INTEGER is
      local
         r: INTEGER
      do
         r := posix_pathconf (sh.string_to_pointer (directory_name), PC_NAME_MAX)
         if r = -1 then
            raise_posix_error
         end
         Result := r
      end
   
   
feature

   dispose is
      do
         if is_open then
            close
         end
      end


feature {NONE} -- state
   
   directory_name: POSIX_PATH

   dirp: POINTER


invariant

   valid_directory_name: directory_name /= Void

end -- class POSIX_DIRECTORY
