indexing

   description: "Test Standard C file class."

   Bugs: "1. Assumes 8 bit characters."
  
   author: "Berend de Boer"
   date: "$Date: Fri, 13 Oct 2000 13:53:08 +0200 $"
   revision: "$Revision: 1.5 $"

class 
   
   TEST_FILE


inherit
   
   STDC_FILE_SYSTEM
      rename
         make as make_file_system
      end
   

creation
   
   make
   
   
feature

   test_text: STRING is "hello world"
   
   make is
      local
         file: STDC_BINARY_FILE
         textfile: STDC_TEXT_FILE
         buf: STDC_DYNAMIC_MEMORY
         pos: STDC_FILE_POSITION
         tmpfile: STDC_TEMPORARY_FILE
      do
         print ("testing common file interface.%N")

         make_file_system

         print ("Create a file for reading and writing.%N")
         create file.create_read_write ("test.berend")
         print ("close the file.%N")
         file.close

         print ("Create a file for reading and writing.%N")
         create file.create_read_write ("test.berend")
         print ("Write something%N")
         file.write_string (test_text)         
         print ("close the file.%N")
         file.close
         
         print ("Create a file for writing.%N")
         create file.create_read_write ("test.berend")
         file.write_string (test_text)
         file.close
         
         print ("Create a file for reading.%N")
         create file.open_read ("test.berend")
         print ("Read a string.%N")
         file.read_string (test_text.count)
         if equal(file.last_string, test_text) then
            print ("read exactly what written.%N")
         else
            print ("!! Expected: ")
            print (test_text)
            print (", got: '")
            print (file.last_string)
            print ("'.%N")
         end
         file.close
         
         print ("Open existing file with read-only access.%N")
         create file.open_read ("test.berend")

         print ("Test random access. Create a file.%N")
         create file.create_read_write ("test.berend")
         print ("Write three integers.%N")
         file.write_string ("10")
         file.write_string ("20")
         file.write_string ("30")
         print ("Current position: ")
         print (file.tell.out)
         print ("%N")
         print ("Go to 2nd integer with seek.%N")
         file.seek (2)
         print ("Read it.%N")
         file.read_string (2)
         check_if_20 (file.last_string)
         print ("Read 2nd integer again with seek_from_current.%N")
         file.seek_from_current (-2)
         file.read_string (2)
         check_if_20 (file.last_string)
         print ("Current position: ")
         print (file.tell.out)
         print ("%N")
         print ("Read 2nd integer again with seek_from_end.%N")
         file.seek_from_end (-4)
         file.read_string (2)
         check_if_20 (file.last_string)
         print ("Current position: ")
         print (file.tell.out)
         print ("%N")

         print ("Test error.%N")
         if file.error then
            print ("Error set.%N")
         else
            print ("Error not set.%N")
         end
         print ("Clear error indicator.%N")
         file.clearerr

         print ("Flush.%N")
         file.flush

         print ("Test writing and reading bytes.%N")
         create file.create_read_write ("test.berend")
         print ("Read something from an empty file.%N")
         file.getc
         print ("Test eof.%N")
         if file.eof then
            print ("EOF as expected.%N")
         else
            print ("!! not EOF.%N")
         end
         print ("Write a byte.%N")
         file.putc (65)
         file.rewind
         file.getc
         if file.last_byte /= 65 then
            print ("!! Expected 65, got: ")
            print (file.last_byte)
            print ("%N")
         else
            print ("Got 65 as expected.%N")
         end

         print ("Reopen the same file.%N")
         file.reopen ("test.berend", "w")

         print ("Test buffering.%N")
         create file.open ("test.berend", "r")
         create buf.allocate (1024)         
         file.setbuf (buf.ptr)
         file.set_full_buffering (buf.ptr, buf.size)
         file.set_line_buffering (buf.ptr, buf.size)
         file.set_no_buffering

         print ("Test ungetc.%N")
         create file.create_read_write ("test.berend")
         file.ungetc (65)
         file.getc
         if file.last_byte /= 65 then
            print ("!! Expected 65, got: ")
            print (file.last_byte)
            print ("%N")
         else
            print ("Got 65 as expected.%N")
         end
         file.close

         print ("Open a file for appending.%N")
         create file.open_append ("test.berend")
         print ("close the file.%N")
         file.close

         print ("Test file positioning.%N")
         create textfile.create_read_write ("test.berend")
         textfile.write_string ("hello")
         pos := textfile.get_position
         textfile.write_string ("world")
         textfile.set_position (pos)
         textfile.read_string (128)
         if not equal (textfile.last_string, "world") then
            print ("!! ")
         end
         print ("String read: ")
         print (textfile.last_string)
         print ("%N")
         textfile.close

         remove_file ("test.berend")

         print ("Create a temporary file.%N")
         create tmpfile.make
         tmpfile.puts ("hello%N")
         tmpfile.close

         print ("End of common file test.%N")
      end

feature   

   check_if_20 (s: STRING) is
      do
         if equal(s, "20") then
            print ("Got the right string.%N")
         else
            print ("!! Expected 20, got: ")
            print (s)
            print (".%N")
         end
      end

end -- class TEST_FILE
